------------------------------------------------------------------------------
-- SellableItemDrops - Logging your sellable item drops (TSM3 Module) 
------------------------------------------------------------------------------
-- Modules/Data.lua - data gathering, loading, saving
--
-- Author: Caraxe
--
-- Version 0.3.6
------------------------------------------------------------------------------

local addonName, addon = ...
local Data = addon:NewModule("Data", "AceEvent-3.0", "AceHook-3.0", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local LibParse = LibStub("LibParse")
local private = {}

DATADEBUG = nil

-- blacklisted items
-- AQ20
private.BLACKLISTED_ITEMS = "i:18640,i:19183,i:20858,i:20859,i:20860,i:20861,i:20862,i:20863,i:20864,i:20865,i:20866," ..
		"i:20867,i:20868,i:20869,i:20870,i:20871,i:20872,i:20873,i:20874,i:20875,i:20876,i:20877,i:20878,i:20879," ..
		"i:20881,i:20882,"
-- Zul Aman
private.BLACKLISTED_ITEMS = private.BLACKLISTED_ITEMS .. "i:33865,"
-- Zul Farrak
private.BLACKLISTED_ITEMS = private.BLACKLISTED_ITEMS .. "i:8623,"
-- Dreanor
private.BLACKLISTED_ITEMS = private.BLACKLISTED_ITEMS .. "i:120945,i:118897,i:118903,"
-- Silithus
private.BLACKLISTED_ITEMS = private.BLACKLISTED_ITEMS .. "i:20406,i:20407,i:20408,"

private.sesGoldUpdated = false
private.sesItemsUpdated = false

function Data:ToggleDebug()
	addon.isDebug = not addon.isDebug
	if (not addon.isDebug) then
		addon:Printf(L["Debug is off"])
	else
		addon.Data:DebugPrintf(L["Debug is on"])
	end
end

function Data:DebugPrintf(...)
	if (addon.isDebug) then
		Data:Printf(...)
	end
end

function Data:InsertItem(link, quantity, rarity)
	if not (link and quantity) then return end

	-- trigger Item Cache
	TSMAPI.Item:FetchInfo(link)
	
	local record = {}
	record.link = link
	record.quantity = quantity
	record.time = time()
	record.player = UnitName("player")
	if (addon.isDemo) then
		record.player = "R2-D2"
	end
	record.zone = GetZoneText()
	
	if not addon.sesItems[link] then
		addon.sesItems[link] = {drops={}}
		addon.sesItems[link].iString = TSMAPI.Item:ToItemString(link)
		-- addon.sesItems[link].iName = TSMAPI.Item:GetName(addon.sesItems[link].iString) or "?"
		-- addon.sesItems[link].iRarity = TSMAPI.Item:GetQuality(addon.sesItems[link].iString) or 0
		addon.sesItems[link].iBaseString = TSMAPI.Item:ToBaseItemString(link)
		-- addon.sesItems[link].iBaseName = TSMAPI.Item:GetName(addon.sesItems[link].iBaseString) or "?"
		-- always real item!
		-- addon.sesItems[link]["mv" .. addon.db.global.mvSource] = Data:GetItemValue(addon.sesItems[link].iString)
	end
	tinsert(addon.sesItems[link].drops, record)
	sort(addon.sesItems[link].drops, function(a, b) return (a.time or 0) < (b.time or 0) end)
end

function Data:PurgeItem(data, items)
	local qty = 0
	if (data) then
		if (data.link) and (data.time) then
			addon.Data:DebugPrintf(L["Purging %s, dropped at %s."], data.link, data.time)
			if (items[data.link]) and (items[data.link].drops) then
				local p
				for i, record in ipairs(items[data.link].drops) do
					addon.Data:DebugPrintf(L["%s: Drop at %s"], i, record.time)
					if (data.time == record.time) then
						p = i
						qty = record.quantity
					end
				end
				if (p) then
					addon.Data:DebugPrintf(L["Purged %sx %s at %s."], qty, data.link, data.time)
					tremove(items[data.link].drops,p)
				end
			end
		end
	end
	return qty
end

function Data:Load()
	Data:LoadSession()
end

function Data:LoadFromCSV(csv)
	local items = {}
	
	if not csv then
		return items
	end
	
	local cnt = 0
	for _, record in ipairs(select(2, LibParse:CSVDecode(csv)) or {}) do
		if record.link and type(record.time) == "number" then
			
			--addon:Printf("Loading data for %s: itemString %s baseItemString %s", 
			--	record.link, 
			--	TSMAPI.Item:ToItemString(record.link),
			--	TSMAPI.Item:ToBaseItemString(record.link))
			
			-- trigger Item Cache
			TSMAPI.Item:FetchInfo(record.link)

			-- base item info
			if not items[record.link] then
				items[record.link] = {drops={}}
				items[record.link].iString = TSMAPI.Item:ToItemString(record.link)
				-- items[record.link].iName = TSMAPI.Item:GetName(items[record.link].iString) or "?"
				-- items[record.link].iRarity = TSMAPI.Item:GetQuality(items[record.link].iString) or 0
				items[record.link].iBaseString = TSMAPI.Item:ToBaseItemString(record.link)
				-- items[record.link].iBaseName = TSMAPI.Item:GetName(items[record.link].iBaseString) or "?"
				-- always real item!
				-- items[record.link]["mv" .. addon.db.global.mvSource] = Data:GetItemValue(items[record.link].iString)
			end
			
			-- drop related info
			record.zone = record.zone or L["Unknown"]
			record.quantity = record.quantity or 1
			record.player = record.player or L["Unknown"]
			if (addon.isDemo) then
				record.player = "R2-D2"
			end
			record.time = record.time or 0
			record.itemString = record.link -- TODO: rename to link
			
			tinsert(items[record.link].drops, record)
			cnt = cnt + 1
		end
	end
	
	for link in ipairs(items) do
		sort(items[link].drops, function(a, b) return (a.time or 0) < (b.time or 0) end)
	end
	
	addon.Data:DebugPrintf(L["Loading data for %s item drops."], cnt)
	return items
end

function Data:LoadFromBag(id, items)
	addon.Data:DebugPrintf(L["LoadFromBag(): Bag #%s."], id)
	items = items or {}
	local cnt = 0

	for j = 1, GetContainerNumSlots(id) do
		addon.Data:DebugPrintf(L["Bag #%s - %s."], id, j)
		local _, quantity, locked, quality, readable, lootable, link, isFiltered, noValue, itemID = GetContainerItemInfo(id,j);
		
		if (link) then
			-- trigger Item Cache
			TSMAPI.Item:FetchInfo(link)

			local record = {}
			record.link = link
			record.quantity = quantity or 1
			record.zone = L["Bag "] .. id
			record.player = UnitName("player")
			if (addon.isDemo) then
				record.player = "R2-D2"
			end
			record.time = 0
			record.itemString = record.link -- TODO: rename to link
			
			if not items[record.link] then
				items[record.link] = {drops={}}
				items[record.link].iString = TSMAPI.Item:ToItemString(record.link)
				-- items[record.link].iName = TSMAPI.Item:GetName(items[record.link].iString) or "?"
				-- items[record.link].iRarity = TSMAPI.Item:GetQuality(items[record.link].iString) or 0
				items[record.link].iBaseString = TSMAPI.Item:ToBaseItemString(record.link)
				-- items[record.link].iBaseName = TSMAPI.Item:GetName(items[record.link].iBaseString) or "?"
				-- always real item!
				--items[record.link]["mv" .. addon.db.global.mvSource] = Data:GetItemValue(items[record.link].iString)
			end
		
			tinsert(items[record.link].drops, record)
			cnt = cnt + 1
		end
	end
	
	addon.Data:DebugPrintf(L["Loading %s items from bag %s."], cnt, id)
	return items
end

function Data:LoadFromBags(items)
	items = items or {}
	for i = 0, NUM_BAG_SLOTS do
		items = Data:LoadFromBag(i, items)
	end
	return items
end

function Data:UpdateInventory()
	addon.Data:DebugPrintf(L["UpdateInventory()"])

	addon.invInfo = addon.invInfo or {}
	addon.invInfo.invTime = 0
	addon.invInfo.invZone = nil
	addon.invInfo.invEnd = 0
	addon.invInfo.invMV = 0
	addon.invInfo.invMVSource = addon.db.global.mvSource

	addon.invItems = addon.invItems or {}
	local c = 0
	for _, data in pairs(addon.invItems) do
		for _, record in ipairs(data.drops) do
			c = c + 1
			if record.time > addon.invInfo.invEnd then
				addon.invInfo.invEnd = record.time
			end
			addon.invInfo.invZone = addon.invInfo.invZone or record.zone
			local lItemString = data.iString
			if (addon.db.global.useNotTheRealItemString) then
				lItemString = data.iBaseString
			end
			local marketValue = addon.Data:GetItemValue(lItemString) or 0
			addon.invInfo.invMV = addon.invInfo.invMV + (marketValue * record.quantity)
		end
	end
	addon.Data:DebugPrintf(L["UpdateInventory: %s items"], c)
end


function Data:LoadSession()
	addon.sesItemsLoaded = false
	addon.sesItems = Data:LoadFromCSV(addon.db.realm.csvDrops)
	addon.sesItemsLoaded = true

	if addon.db.realm.lapNum == 0 then
		addon.db.realm.lapNum = 1 
	end
	if (addon.db.realm.lapTime == 0) then
		addon.db.realm.lapTime = time()
	end
	if (addon.db.realm.sesTime == 0) then
		addon.db.realm.sesTime = time()
	end
end

function Data:SaveToCSV(items)
	local DROP_KEYS = { "link", "player", "quantity", "time", "zone"}
	local drops = {}

	for itemString, data in pairs(items) do
		for _, record in ipairs(data.drops) do
			local link = gsub(record.link, ",", ";")
			if (link ~= record.link) then
				addon:Printf("Saving Item %s without ','?!", record.link)
				record.link = link
			end
			tinsert(drops, record)
		end
	end

	return LibParse:CSVEncode(DROP_KEYS, drops)
end

function Data:SaveSession()
	-- example:
	-- |cff0070dd|Hitem:127665:0:0:0:0:0:0:0:100:66:0:0:1:41|h[Wickelschuppenbänder]|h|r,<char>,1,1449105939,Tanaandschungel
	addon.db.realm.csvDrops = Data:SaveToCSV(addon.sesItems)
end

function Data:LoadDatabaseTag(tag)
	addon.hisItems = addon.hisItems or {}
	addon.hisItems[tag] = Data:LoadFromCSV(addon.db.realm.csvHis[tag])
	Data:UpdateDatabaseTag(tag)
end

function Data:SaveDatabaseTag(tag)
	addon.db.realm.csvHis = addon.db.realm.csvHis or {}
	addon.db.realm.csvHis[tag] = Data:SaveToCSV(addon.hisItems[tag])
end

function Data:UpdateDatabaseTag(tag)
	addon.hisInfo = addon.hisInfo or {}
	
	if (tag == 0) then
		return
	end
	
	addon.hisInfo[tag] = addon.hisInfo[tag] or {}
	
	addon.Data:DebugPrintf(L["UpdateDatabase() for %s"], tag)
		
	addon.hisInfo[tag].hisTime = 0
	addon.hisInfo[tag].hisZone = nil
	addon.hisInfo[tag].hisEnd = 0
	addon.hisInfo[tag].hisMV = 0
	addon.hisInfo[tag].hisMVSource = addon.db.global.mvSource

	local c = 0
	for _, data in pairs(addon.hisItems[tag]) do
		for _, record in ipairs(data.drops) do
			c = c + 1
			if record.time > addon.hisInfo[tag].hisEnd then
				addon.hisInfo[tag].hisEnd = record.time
			end
			if (addon.hisInfo[tag].hisTime == 0) then
				addon.hisInfo[tag].hisTime = record.time
			end
			if record.time < addon.hisInfo[tag].hisTime then
				addon.hisInfo[tag].hisTime = record.time
			end
			addon.hisInfo[tag].hisZone = addon.hisInfo[tag].hisZone or record.zone
			local lItemString = data.iString
			if (addon.db.global.useNotTheRealItemString) then
				lItemString = data.iBaseString
			end
			local marketValue = addon.Data:GetItemValue(lItemString) or 0
			addon.hisInfo[tag].hisMV = addon.hisInfo[tag].hisMV + (marketValue * record.quantity)
		end
	end
	addon.Data:DebugPrintf(L["UpdateDatabase: %s items"], c)
end

function Data:WipeHistoryTag(tag)
	addon.hisItems = addon.hisItems or {}
	addon.db.realm.csvHis = addon.db.realm.csvHis or {}
	if (tag > 0) then
		addon.hisItems[tag] = nil
		addon.db.realm.csvHis[tag] = nil
		addon:Printf(L["Deleting session %s."], date(addon.timeFormat, tag))
	end
end

function Data:ResetCurrentTag(flCurrent)
	flCurrent = flCurrent or false
	local newTag = 0
	for tag in pairs(addon.hisItems) do
		if (tag > newTag) then
			newTag = tag
		end
	end
	addon.hisCTag = addon.hisCTag or 0
	if (addon.hisCTag == 0) or (flCurrent) then
		addon.hisCTag = newTag
	end
	addon.Data:DebugPrintf(L["ResetCurrentTag() addon.hisCTag=%s"], addon.hisCTag)
end

function Data:LoadDatabase()
	if not addon.hisItems then
		addon:Printf(L["Loading Session History..."])
		addon.hisItems = {}
	end
	
	for tag in pairs(addon.db.realm.csvHis) do
		if (addon.hisItems[tag]) then
			addon.Data:DebugPrintf(L["Session %s already loaded."], date(addon.timeFormat, tag))
		else
			addon.Data:DebugPrintf(L["Loading session %s."], date(addon.timeFormat, tag))
			Data:LoadDatabaseTag(tag)
		end
	end
	Data:ResetCurrentTag()
end

function Data:SessionToDatabase()
	-- session data becomes history data
	addon.hisItems = addon.hisItems or {}
	local tag = addon.db.realm.sesTime
	-- overwriting previous tag entry
	if (addon.hisItems[tag]) then
		addon.hisItems[tag] = nil
	end
	addon.hisItems[tag] = addon.sesItems
	-- history data becomes csv data
	addon.db.realm.csvHis = addon.db.realm.csvHis or {}
	addon.db.realm.csvHis[tag] = Data:SaveToCSV(addon.hisItems[tag])
	--addon.db.realm.hisIndex = addon.db.realm.hisIndex or {}
	--addon.db.realm.hisIndex[tag] = 1
	addon:Printf(L["Saving session %s."], date(addon.timeFormat, tag))
	Data:UpdateDatabaseTag(tag)
end

function Data:WipeSession()
	addon.db.realm.sesMV = 0 
	addon.db.realm.sesGold = 0 
	addon.db.realm.sesNumItems = 0 
	addon.db.realm.sesNumNW = 0 
	addon.db.realm.sesTime = time()
	addon.db.realm.sesEnd = 0
	addon.db.realm.lapTime = time()
	addon.db.realm.lapNum = 1
	addon.sesItems = {}
end

function Data:GetItemValue(itemString)
	local mvCopper = TSMAPI:GetItemValue(itemString, addon.db.global.mvSource) or 0
	-- FIXME: for custom sources TSMAPI:GetCustomPriceValue(addon.db.global.mvSource, itemString) should be used

	local oldmvCopper = mvCopper
	-- Grays
	local itemRarity = TSMAPI.Item:GetQuality(itemString) or 0
	if (itemRarity == 0) then
		if (addon.db.global.useVendorSellForGrays) then
			mvCopper = TSMAPI:GetItemValue(itemString, "VendorSell") or 0
		end
	else
		if (mvCopper == 0) and (addon.db.global.useVendorSellForBOP) then
			-- use the real itemString not the itemID in case there are item variations/upgrades
			mvCopper = TSMAPI:GetItemValue(itemString, "VendorSell") or 0
		end
	end
	if (mvCopper == oldmvCopper) then
	--	addon.Data:DebugPrintf(L["GetItemValue (%s) for %s is %s"], addon.db.global.mvSource, itemString, 
	--		GetCoinTextureString(mvCopper))
	else
	--	addon.Data:DebugPrintf(L["GetItemValue (%s) for %s is %s, but using VendorSell value %s instead"], 
	--		addon.db.global.mvSource, 
	--		itemString, 
	--		GetCoinTextureString(oldmvCopper),
	--		GetCoinTextureString(mvCopper)
	--	)
	end
	
	return mvCopper
end

local IsProfessionBag
do
	local bagTypes = {
		-- 1048576: Tackle Box
		[0x100000] = true,
		-- 65536: Cooking Bag
		[0x10000] = true,
		-- 1024: Mining Bag
		[0x0400] = true,
		-- 512: Gem Bag
		[0x0200] = true,
		-- 128: Engineering Bag
		[0x0080] = true,
		-- 64: Enchanting Bag
		[0x0040] = true,
		-- 32: Herb Bag
		[0x0020] = true,
		-- 16: Inscription Bag
		[0x0010] = true,
		-- 8: Leatherworking Bag
		[0x0008] = true,
	}
	IsProfessionBag = function(bagType)
		return bagTypes[bagType] or false
	end
end

function Data:GetFreeBagSlots()
	local freeSlots = 0
	local totalSlots = 0
	local takenSlots = 0
	for i = 0, NUM_BAG_SLOTS do
		local usable = true
		local freeSlots, bagType = GetContainerNumFreeSlots(i)
		if i >= 1 then
			if IsProfessionBag(bagType) then
				usable = false
			end
		end
		if usable then
			local bagSize = GetContainerNumSlots(i)
			if bagSize and bagSize > 0 then
				totalSlots = totalSlots + bagSize
				takenSlots = takenSlots + (bagSize - freeSlots)
			end
		end
	end
	freeSlots = totalSlots - takenSlots
	return freeSlots
end

-- current implementation is based on event LOOT_OPENED and it works un-grouped, 
-- without enought free bag space the addition of items is postponed until there is space 
-- update of the item lists is done only out-of-combat to avoid script-ran-too-long errors
local function ProcessEvent(event, ...)
	addon.Data:DebugPrintf(L["Event %s"], event)
	if event == "LOOT_OPENED" then
		if (addon.db.realm.sesEnd == 0) then
			for i = 1, GetNumLootItems() do
				local slotType = GetLootSlotType(i);
				if slotType == 1 then
					-- items
					local itemLink = GetLootSlotLink(i)
					local itemQty, itemLRarity, isLocked = select(3, GetLootSlotInfo(i))
					-- FIXME: look at isLocked
					if (itemLink) then
						-- Query free bag space check at this moment solves not the loot situation, 
						-- perhape we have to go the bag-snapshot-way
						-- but if there are not any free slots, there is really no point in adding the item to session data
						if (Data.GetFreeBagSlots() < 1) then
							addon:Printf(L["Probabliy no free bag space for %s."], itemLink)
						else
							-- find out, if we use base item or not
							local itemString = TSMAPI.Item:ToItemString(itemLink)
							if (addon.db.global.useNotTheRealItemString) then
								itemString = TSMAPI.Item:ToBaseItemString(itemLink)
							end
							if (itemString == nil) then
								addon:Printf(L["Call to TSMAPI.Item:ToItemString('%s') failed."], itemLink)
							else
								local mvCopper = Data:GetItemValue(itemString)
								if (itemLRarity >= addon.db.global.minRarity) then
									if (Data:IsBlacklisted(itemString)) then
										if (itemQty > 1) then
											addon:Printf(L["#%s. %sx%s (%s): %s (blacklisted)"], i, itemLink, itemQty, itemString, GetCoinTextureString(mvCopper * itemQty))
										else
											addon:Printf(L["#%s. %s (%s): %s (blacklisted)"], i, itemLink, itemString, GetCoinTextureString(mvCopper * itemQty))
										end
									else
										addon.db.realm.sesMV = addon.db.realm.sesMV + (mvCopper * itemQty)
										addon.db.realm.sesNumItems = addon.db.realm.sesNumItems + itemQty
										if TSMAPI:MoneyFromString(addon.db.global.minNoteworthy) then
											if (mvCopper > TSMAPI:MoneyFromString(addon.db.global.minNoteworthy)) then
											addon.db.realm.sesNumNW = addon.db.realm.sesNumNW + itemQty
												if (addon.db.global.nwSound ~= "") then
													PlaySound(addon.db.global.nwSound, "master")
												end
											end
										end
										if (itemQty > 1) then
											addon:Printf(L["#%s. %sx%s (%s): %s"], i, itemLink, itemQty, itemString, GetCoinTextureString(mvCopper * itemQty))
										else
											addon:Printf(L["#%s. %s (%s): %s"], i, itemLink, itemString, GetCoinTextureString(mvCopper * itemQty))
										end
										-- save always the real, not the base item
										Data:InsertItem(itemLink, itemQty, itemLRarity)
										private.sesItemsUpdated = true
									end
								else
									if (itemQty > 1) then
										addon.Data:DebugPrintf(L["#%s. %sx%s (%s): %s"], i, itemLink, itemQty, itemString, GetCoinTextureString(mvCopper * itemQty))
									else
										addon.Data:DebugPrintf(L["#%s. %s (%s): %s"], i, itemLink, itemString, GetCoinTextureString(mvCopper * itemQty))
									end
								end
							end
						end
					end
				elseif slotType == 2 then
					-- money
					local cuString = select(2, GetLootSlotInfo(i))
					local cuVal = {}
					local cuCnt = 0;
					cuString:gsub("%d+", 
						function(i)
							table.insert(cuVal, i)
							cuCnt = cuCnt + 1
						end
					)
					local cuSum = 0
					if (cuCnt == 1) then
						-- raid boss gold detection
						cuString:gsub("(%d+) Gold", 
							function(i)
								cuVal[1] = i * 10000
							end
						)
					end
					if cuCnt == 3 then
						cuSum = (cuVal[1] * 10000) + (cuVal[2] * 100) + cuVal[3]
					elseif cuCnt == 2 then
						cuSum = (cuVal[1] * 100) + cuVal[2]
					else
						cuSum = cuVal[1]
					end
					addon:Printf(L["#%s. %s"], i, GetCoinTextureString(cuSum))
					addon.Data:DebugPrintf("cuString=%s", cuString)
					addon.db.realm.sesGold = addon.db.realm.sesGold + cuSum
					private.sesGoldUpdated = true
				end
			end
			if not (addon.isInfight) then
				if (private.sesGoldUpdated or private.sesItemsUpdated) then
					addon.Data:DebugPrintf("ProcessEvent: UpdateSessionData")
					addon.GUI:UpdateSessionData(private.sesItemsUpdated)
					private.sesGoldUpdated = false
					private.sesItemsUpdated = false
				end
			end
		else
			addon.Data:DebugPrintf(L["Session is not running, so there's no reason to look at the loot."])
		end
	elseif event == "PLAYER_REGEN_DISABLED" then
		-- updating GUI is done only out of fight for performance reasons and to avoid script-run-too-long errors)
		addon.isInfight = true
		addon.Data:DebugPrintf("ProcessEvent: Player enters combat")
	elseif event == "PLAYER_REGEN_ENABLED" then
		addon.isInfight = false
		addon.Data:DebugPrintf("ProcessEvent: Player leaves combat")
		if (private.sesGoldUpdated or private.sesItemsUpdated) then
			addon.Data:DebugPrintf("ProcessEvent: UpdateSessionData (deferred)")
			addon.GUI:UpdateSessionData(private.sesItemsUpdated)
			private.sesGoldUpdated = false
			private.sesItemsUpdated = false
		end
	end
end		

function Data:RegisterLootEvents()
	self:RegisterEvent("LOOT_OPENED", ProcessEvent)
	self:RegisterEvent("PLAYER_REGEN_ENABLED", ProcessEvent)
	self:RegisterEvent("PLAYER_REGEN_DISABLED", ProcessEvent)
	self:RegisterEvent("LOOT_SLOT_CLEARED", ProcessEvent) -- only for monitoring
	self:RegisterEvent("LOOT_READY", ProcessEvent) -- only for monitoring
end

function Data:IsBlacklisted(itemString)
	local remString = ""
	if not private.blItems then
		private.blItems = {}
		local i = 0
		private.BLACKLISTED_ITEMS:gsub("(i:[^,]+)", 
			function(id)
				local lItemString = TSMAPI.Item:ToItemString(id)
				if (lItemString) then
					private.blItems[lItemString] = true
					i = i + 1
					local name = TSMAPI.Item:GetName(id) 
					local link = TSMAPI.Item:GetLink(id)
					if (name) then
						remString = remString .. " " .. link
					else 
						remString = remString .. " " .. id
					end
				end
			end
		)
		addon:Printf(L["Imported %s blacklisted items: %s"], i, remString)
	end
	
	if (itemString ~= "") then
		local lItemString = TSMAPI.Item:ToBaseItemString(itemString)
		if (lItemString) then
			if (private.blItems[lItemString]) then
				-- addon.Data:DebugPrintf(L["Item %s (%s) is blacklisted."], lItemString, itemString)
				return true
			end
		end
		-- addon.Data:DebugPrintf(L["Item %s (%s) is not blacklisted."], lItemString, itemString)
	end
	return false
end

-- EOF
