------------------------------------------------------------------------------
-- SellableItemDrops - Logging your sellable item drops (TSM3 Module) 
------------------------------------------------------------------------------
-- SellableItemDrops.lua 
--
--[[
SellableItemDrops remembers your looted items.
A list shows name, time, location of every dropped item and it's market value 
based on a selectable Trade Skill Master price source.

The current session data can be archived to a database and this database of 
previous sessions can be searched for items and more.

SellableItemDrops is useful during competitive transmog challenge runs or 
farming instances to observe the accumulated market value of all dropped items 
in a given time period.

The Session Tab and the History Tab opens with "/tsm sid" as they are 
Trade Skill Master v3 modules. Addon options are configured through the 
TSM3 Module Options Tab "SellableItemDrops".

It runs nicely together with Loot Appraiser and Loot Appraiser Challenge, 
so you can search your dropped items and take screenshots to compare it with others.

Requirements:

SellableItemDrops requires TradeSkillMaster v3. To use the default price source DBGlobalMarketAvg 
the additional addons TradeSkillMaster_AuctionDB, TradeSkillMaster_AppHelper and 
a working TSM Desktop App are needed.

Author: Caraxe
All Rights Reserved

Version 0.3.6
------------------------------------------------------------------------------]]

local addonName, addon = ...
addon = LibStub("AceAddon-3.0"):NewAddon(addon, addonName, "AceEvent-3.0", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local LibParse = LibStub("LibParse")
local addonShortName = "sid"

local LA = LibStub("AceAddon-3.0"):GetAddon("LootAppraiser",true)
local CHALLENGE = LibStub("AceAddon-3.0"):GetAddon("LootAppraiser Challenge",true)

local settingsInfo = {
	version = 13,
	global = {
		timeFormat = { type = "string", default = "ago", lastModifiedVersion = 2 },
		mvSource = { type = "string", default = "DBGlobalMarketAvg", lastModifiedVersion = 1 },
		mvCustomSource = { type = "string", default = "DBGlobalMarketAvg", lastModifiedVersion = 6 },
		minRarity = { type = "number", default = 2, lastModifiedVersion = 1 },
		minNoteworthy = { type = "string", default = "1000g", lastModifiedVersion = 13 },
		useVendorSellForBOP = { type = "boolean", default = true, lastModifiedVersion = 1 },
		useVendorSellForGrays = { type = "boolean", default = true, lastModifiedVersion = 1 },
		useNotTheRealItemString = { type = "boolean", default = true, lastModifiedVersion = 1 },
		nwSound = { type = "string", default = "AuctionWindowOpen", lastModifiedVersion = 1 },
		frScale = { type = "boolean", default = false, lastModifiedVersion = 1 },
		lpInstanceReset = { type = "boolean", default = true, lastModifiedVersion = 13 },
		shPlayer = { type = "boolean", default = true, lastModifiedVersion = 11 },
	},
	realm = {
		csvDrops = { type = "string", default = "", lastModifiedVersion = 1 },
		csvHis = { type = "table", default = {}, lastModifiedVersion = 5 },
		sesMV = { type = "number", default = 0, lastModifiedVersion = 3 },
		sesGold = { type = "number", default = 0, lastModifiedVersion = 3 },
		sesNumItems = { type = "number", default = 0, lastModifiedVersion = 3 },
		sesNumNW = { type = "number", default = 0, lastModifiedVersion = 3 },
		sesTime = { type = "number", default = 0, lastModifiedVersion = 3 },
		sesEnd = { type = "number", default = 0, lastModifiedVersion = 7 },
		lapNum = { type = "number", default = 1, lastModifiedVersion = 7 },
		lapTime = { type = "number", default = 0, lastModifiedVersion = 7 },
	},
}

function addon:OnInitialize()
	addon.db = TSMAPI.Settings:Init(addonName .. "DB", settingsInfo)
	addon.isDebug = false
	addon.isDemo = false
	addon.isInfight = false
	addon.version = GetAddOnMetadata(addonName, "Version")
	
	-- for shorter access with addon.Data etc.
	for module in pairs(addon.modules) do
		addon[module] = addon.modules[module]
	end
	addon:RegisterModule()

	-- LA integration
	if LA then
		addon:Printf(L["Found %s Version %s."], LA.METADATA.NAME, LA.METADATA.VERSION)
	end
	if CHALLENGE then
		addon:Printf(L["Found %s Version %s."], CHALLENGE.METADATA.NAME, CHALLENGE.METADATA.VERSION)
	end
	
	addon.timeFormat = "%Y-%m-%d %H:%M:%S";
	if (GetLocale() == "deDE") then
		addon.timeFormat = "%d.%m.%Y %H:%M:%S"
	end
end

local function SecTimer()
--	addon:Printf(L["Timertick"] .. " " .. tostring(time()))
	local oldDebug = addon.isDebug
	addon.isDebug = false
	addon.GUI:UpdateSessionData(false)
	addon.isDebug = oldDebug
end

function addon:CheckSettings()
	-- Check Price Sources
	-- local psFallBack = "VendorSell" -- use fallback
	local psFallBack = "" -- don't use fallback
	local mvSourcesList = TSMAPI:GetPriceSources() or {}
	if not mvSourcesList then
		addon:Printf(L["|cffff0000IMPORTANT:|r"])
		addon:Printf(L["TSM returned no price information. Please enable TSM and/or check your price source."])
	else
		if (addon.db.global.mvSource == "Custom") then
			local mv, err = TSMAPI:ValidateCustomPrice(addon.db.global.mvCustomSource)
			if not mv then
				addon:Printf(L["|cffff0000IMPORTANT:|r"])
				addon:Printf(L["Your custom price source '%s' is currently not usable."], addon.db.global.mvCustomSource)
				addon:Printf(L["TSM returned the following error: %s."], err)
				if (psFallback ~= "") then
					addon:Printf(L["'%s' is used for now."], psFallBack)
					addon.db.global.mvSource = psFallBack
				end
			end
		else
			found = false
			for k,v in pairs(mvSourcesList) do
				addon.Data:DebugPrintf("found TSM Price Source %s/%s", k, v)
				if k == addon.db.global.mvSource then
					addon.Data:DebugPrintf(" ... using it")
					found = true
					break
				end
			end
			if not found then
				addon:Printf(L["|cffff0000IMPORTANT:|r"])
				addon:Printf(L["Your price source '%s' is currently not usable."], 
					mvSourcesList[addon.db.global.mvSource] or addon.db.global.mvSource)
				addon:Printf(L["Please enable the right TSM module and/or update your TSM desktop app."])
				if (psFallBack ~= "") then
					addon:Printf(L["'%s' is used for now."], mvSourcesList[psFallBack] or psFallBack)
					addon.db.global.mvSource = psFallBack
				end
			end
		end
	end
	
	if not TSMAPI:MoneyFromString(addon.db.global.minNoteworthy) then
		addon:Printf(L["|cffff0000IMPORTANT:|r Your entry of '%s' for noteworthy items is not usable. It has been reset to 500g."], 
			addon.db.global.minNoteworthy)
		addon.db.global.minNoteworthy = "500g"
	end
end

function addon:OnEnable()
	addon.Data:Load()
	addon:UpdateBroker()
	addon.Data:RegisterLootEvents()
	addon.sectimer = C_Timer.NewTicker(1, SecTimer)
	
	addon:CheckSettings()
	addon.GUI:ShowInfo()
	
	if not addon.sesItemsLoaded then
		addon:Printf(L["|cffff0000IMPORTANT:|r"] ..
			L["There was an error loading the previous item data. "] ..
			L["Heads up, recover is possible:\n"] ..
			L[" 1. Logout (Character Screen is ok).\n"] ..
			L[" 2. Find your game data directory WTF\\<Account>>\\SaveVariables\n"] ..
			L[" 3. Delete the file "] .. addonName .. L[".lua and rename "] .. addonName .. L[".bak to"] .. addonName .. L[".lua\n"] ..
			L[" 4. Login. "])
	end
end

function addon:RegisterModule()
	addon.icons = {
		{ 
			side = "module", 
			desc = "Sellable Item Drops", 
			slashCommand = addonShortName, 
			callback = "GUI:Load", 
			icon = "Interface\\Icons\\INV_Misc_Bag_10_Blue" 
		},
	}
	addon.moduleOptions = { 
		callback = "Options:Load" 
	}
	addon.slashCommands = {
		{ key = addonShortName .. "debug", 
			label = L["Toggles debug mode of Sellable Item Drops."], 
			callback = function(...) 
				addon.Data:ToggleDebug()
				addon.GUI:ToggleDebug()
			end, },
	}

	TSMAPI:NewModule(addon)
end

function addon:OnTSMDBShutdown()
	addon.Data:SaveSession()
end

-----------------------------------------------------------------------------
-- LDB Integration

addon.LDB = LibStub("LibDataBroker-1.1"):NewDataObject(addonName, {
	type = "data source", -- data source
	label = addonName,
	text = L["Loading"],
	icon = "Interface\\Icons\\INV_Misc_Bag_10_Blue", 
	OnClick = function(self, button, down)
		if button == "LeftButton" then
			-- open session tab
			local editbox = ChatEdit_ChooseBoxForSend(DEFAULT_CHAT_FRAME)
			ChatEdit_ActivateChat(editbox)
			editbox:SetText("/tsm " .. addonShortName)
			ChatEdit_OnEnterPressed(editbox)
		elseif button == "RightButton" then
			if (addon.db.realm.sesEnd == 0) then
				-- start next lap
				addon.db.realm.lapTime = time()
				addon.db.realm.lapNum = addon.db.realm.lapNum + 1
				if (addon.db.global.nwSound ~= "") then
					PlaySound(addon.db.global.nwSound, "master")
				end
				if (addon.db.global.lpInstanceReset) then
					ResetInstances()
					addon:Printf(L["Resetting the instance."])
				end
				addon:Printf(L["Finishing Lap #%s with Looted Item Value %s."],
				addon.db.realm.lapNum - 1, GetCoinTextureString(addon.db.realm.sesMV))
			end
		end
	end,
	OnTooltipShow = function(tt)
		tt:AddLine(addonName .. " " .. addon.version)
		tt:AddLine(" ")
		tt:AddLine(L["Current Session"])
		tt:AddDoubleLine(
			format(L["%sStarted at:%s"], ITEM_QUALITY_COLORS[6].hex, FONT_COLOR_CODE_CLOSE), 
				date(addon.timeFormat, addon.db.realm.sesTime))
		if (addon.db.realm.sesEnd == 0) then
			tt:AddDoubleLine(
				format(L["%sDuration:%s"], ITEM_QUALITY_COLORS[6].hex, FONT_COLOR_CODE_CLOSE), 
				SecondsToTime(time() - addon.db.realm.sesTime))
		else
			tt:AddDoubleLine(
				format(L["%sDuration:%s"], ITEM_QUALITY_COLORS[6].hex, FONT_COLOR_CODE_CLOSE), 
				SecondsToTime(addon.db.realm.sesEnd - addon.db.realm.sesTime))
		end
		tt:AddDoubleLine(
			format(L["%sLooted Item Value:%s"], ITEM_QUALITY_COLORS[6].hex, FONT_COLOR_CODE_CLOSE), 
			GetCoinTextureString(addon.db.realm.sesMV))
		tt:AddDoubleLine(
			format(L["%sLooted Gold:%s"], ITEM_QUALITY_COLORS[6].hex, FONT_COLOR_CODE_CLOSE), 
			GetCoinTextureString(addon.db.realm.sesGold))
		local gPerHour = (3600 / (time() - addon.db.realm.sesTime + 1)) * 
			(addon.db.realm.sesMV + addon.db.realm.sesGold) 
		tt:AddDoubleLine(
			format(L["%sGold per Hour:%s"], ITEM_QUALITY_COLORS[6].hex, FONT_COLOR_CODE_CLOSE), 
			GetCoinTextureString(gPerHour))
		tt:AddLine(" ")
		tt:AddLine(format(L["%sLeft-Click%s opens the item database"], ITEM_QUALITY_COLORS[5].hex, FONT_COLOR_CODE_CLOSE))
		if (addon.db.global.lpInstanceReset) then
			tt:AddLine(format(L["%sRight-Click%s starts the next lap and resets the instances"], ITEM_QUALITY_COLORS[5].hex, FONT_COLOR_CODE_CLOSE))
		else
			tt:AddLine(format(L["%sRight-Click%s starts the next lap"], ITEM_QUALITY_COLORS[5].hex, FONT_COLOR_CODE_CLOSE))
		end
	end,
	})

function addon:UpdateBroker()
	local m = addon.db.realm.sesMV or 0
	local gPerHour = (3600 / (time() - addon.db.realm.sesTime + 1)) * 
			(addon.db.realm.sesMV + addon.db.realm.sesGold)
	if (addon.db.realm.sesEnd == 0) then
		if (addon.db.realm.lapNum <= 1) then
			addon.LDB.text = L["Running: "] .. SecondsToTime(time() - addon.db.realm.sesTime) ..  " / " .. 
				"LIV:" .. GetCoinTextureString(m) 
		else
			addon.LDB.text = L["Running (Lap #"] .. addon.db.realm.lapNum .. "): ".. SecondsToTime(time() - addon.db.realm.sesTime) ..  " / " .. 
				"LIV:" .. GetCoinTextureString(m) 
		end
	else
		addon.LDB.text = L["Stopped: "] .. SecondsToTime(addon.db.realm.sesEnd - addon.db.realm.sesTime) ..  " / " .. 
			"LIV:" .. GetCoinTextureString(m) 
	end
end

-- EOF
